<?php
if (!defined('ABSPATH')) {exit;}

/**
 * Fired during plugin activation
 *
 * @link       https://themeforest.net/user/nouthemes/portfolio
 * @since      1.0.0
 *
 * @package    Leopard_Wordpress_Offload_Media
 * @subpackage Leopard_Wordpress_Offload_Media/includes
 */

/**
 * Fired during plugin activation.
 *
 * This class defines all code necessary to run during the plugin's activation.
 *
 * @since      1.0.0
 * @package    Leopard_Wordpress_Offload_Media
 * @subpackage Leopard_Wordpress_Offload_Media/includes
 * @author     Nouthemes <nguyenvanqui89@gmail.com>
 */
class Leopard_Wordpress_Offload_Media_Activator {

	/**
	 * Short Description. (use period)
	 *
	 * Long Description.
	 *
	 * @since    1.0.0
	 */
	public static function activate() {
		self::create_options();
		self::create_files();
		self::create_tables();
		self::unschedule_actions();
		self::create_table_logs();
		self::security_key();
	}

	public static function security_key()
	{
		update_option('nou_leopard_offload_media_security_key', wp_generate_password(30, true));
	}

	public static function unschedule_actions()
	{
		leopard_offload_media_after_action_scheduler_completed();
	}

	public static function create_tables()
	{
		global $wpdb;

		$table_list = array(
			'actionscheduler_actions',
			'actionscheduler_logs',
			'actionscheduler_groups',
			'actionscheduler_claims',
		);

		$found_tables = $wpdb->get_col( "SHOW TABLES LIKE '{$wpdb->prefix}actionscheduler%'" ); // phpcs:ignore WordPress.DB.PreparedSQL.InterpolatedNotPrepared
		foreach ( $table_list as $table_name ) {
			if ( ! in_array( $wpdb->prefix . $table_name, $found_tables ) ) {
				
				$store_schema  = new ActionScheduler_StoreSchema();
				$logger_schema = new ActionScheduler_LoggerSchema();
				$store_schema->register_tables( true );
				$logger_schema->register_tables( true );

				return;
			}
		}
	}

	/**
	 * Create logs table.
	 */
	public static function create_table_logs() {
		require_once ABSPATH . 'wp-admin/includes/upgrade.php';

		global $wpdb;

		$collate = '';

		if ( $wpdb->has_cap( 'collation' ) ) {
			$collate = $wpdb->get_charset_collate();
		}

		// Max DB index length. See wp_get_db_schema().
		$max_index_length = 191;

		$tables = "
		CREATE TABLE {$wpdb->prefix}leopard_offload_stats (
			post_id bigint(20) unsigned NOT NULL,
			date_created datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
			num_items int(11) DEFAULT 0 NOT NULL,
			action varchar(200) NOT NULL,
			PRIMARY KEY (post_id),
			KEY date_created (date_created),
			KEY action (action({$max_index_length}))
		) $collate;";

		dbDelta( $tables );

		if ( $wpdb->get_var( "SHOW TABLES LIKE '{$wpdb->prefix}leopard_offload_stats';" ) ) {
			if ( ! $wpdb->get_var( "SHOW COLUMNS FROM `{$wpdb->prefix}leopard_offload_stats` LIKE 'id';" ) ) {
				$wpdb->query( "ALTER TABLE {$wpdb->prefix}leopard_offload_stats DROP PRIMARY KEY, ADD `id` BIGINT UNSIGNED NOT NULL PRIMARY KEY AUTO_INCREMENT;" );
			}
		}
	}

	/**
	 * Create options
	 *
	 * @since    1.0.0
	 */
	public static function create_options() {
		$default = get_option('nou_leopard_offload_media');
		if(empty($default)){
			$options = array(
				'provider' => 'aws',
				'access_key' => '',
				'secret_access_key' => '',
				'credentials' => ''
				);
			update_option('nou_leopard_offload_media', $options);
			update_option('nou_leopard_offload_media_rewrite_urls_checkbox', 'on');
			update_option('nou_leopard_offload_media_copy_file_s3_checkbox', 'on');
			update_option('nou_leopard_offload_media_private_public_radio_button', 'public');
		}

		$accepted_filetypes = get_option('nou_leopard_offload_media_accepted_filetypes');
		if(empty($accepted_filetypes)){
			update_option('nou_leopard_offload_media_accepted_filetypes', '');
		}

		$cdn = get_option('nou_leopard_offload_media_cdn');
		if(empty($cdn)){
			update_option('nou_leopard_offload_media_cdn', 'default');
		}
	}

	public static function create_files(){
		$files = array(
			array(
				'base'    => LEOPARD_WORDPRESS_OFFLOAD_MEDIA_CACHE_PATH,
				'file'    => '.htaccess',
				'content' => 'deny from all',
			)
		);

		nou_leopard_offload_media_create_files($files);
	}
}
