<?php
if (!defined('ABSPATH')) {exit;}
/**
 * Bunny CDN Client
 *
 * @since      2.0.3
 * @package    Leopard_Wordpress_Offload_Media
 * @subpackage Leopard_Wordpress_Offload_Media/includes
 * @author     Nouthemes <nguyenvanqui89@gmail.com>
 */

class Leopard_Wordpress_Offload_Media_Bunny_Client extends Leopard_Wordpress_Offload_Media_Storage {
    
    private $_storage_key;
    private $_storage_path;

    public $dir = '';

    /**
     * instancia Bunny CDN
     * Bunny CDN constructor.
     */
    public function __construct( $key, $storage_key, $storage_path ) {

        $this->_key = $key;
        $this->_storage_key = $storage_key;
        $this->_storage_path = $storage_path;
    }

    public static function identifier() {
        return 'bunnycdn';
    }

    public static function name() {
        return esc_html__('Bunny CDN', 'leopard-wordpress-offload-media');
    }

    public function Init_S3_Client( $Region='', $Version='', $key='', $Secret='' ) {
        return new Leopard_Wordpress_Offload_Media_BunnyCDN();
    }

    public function Load_Regions() {

        $this->_array_regions = [];
    }

    public function Checking_Credentials() {

        try {

            $S3_Client = $this->Init_S3_Client( '', '', $this->_key );

            $buckets = $S3_Client->Account($this->_key)->GetZoneList();

            update_option( 'nou_leopard_offload_media_connection_success', 1 );

        } catch ( Exception $e ) {

            update_option( 'nou_leopard_offload_media_connection_success', 0 );

            $buckets = 0;

        }

        return $buckets;

    }

    /**
     * obtiene todos los objetos de un bucket
     * @return \Guzzle\Service\Resource\ResourceIteratorInterface|mixed
     */
    public function Show_Buckets($Bucket_Selected='') {

        ob_start();

        if(empty($Bucket_Selected)){
            $Bucket_Selected = ( get_option( 'nou_leopard_offload_media_connection_bucket_selected_select' ) ? get_option( 'nou_leopard_offload_media_connection_bucket_selected_select' ) : '' );
        }

        try {

            // Instantiate the S3 client with your AWS credentials
            $S3_Client = $this->Init_S3_Client( '', '', $this->_key );
            $bucketsStorage = $S3_Client->Account($this->_key)->listStorageZones();
            
            echo "<option value='0'>" . esc_html__( 'Choose a bucket', 'leopard-wordpress-offload-media' ) . "</option>";

            foreach($bucketsStorage['storages'] as $storage){
                               
                if(!isset($storage['Password'])){
                    continue;
                }

                if($storage['Password'] != $this->_storage_key){
                    continue;
                }

                update_option('nou_leopard_offload_media_bunny_region', strtolower($storage['Region']));
                update_option('nou_leopard_offload_media_bunny_zone_list', $storage['PullZones']);

                foreach ( $storage['PullZones'] as $bucket ) {
                    $selected = ( ( $Bucket_Selected == $bucket['Id'] ) ? 'selected="selected"' : '' );
                    ?>
                    <option <?php echo $selected; ?> value="<?php echo esc_attr($bucket['Id']); ?>"> <?php echo esc_html($bucket['Name']); ?> </option>
                    <?php

                }
            }

        } catch ( Exception $e ) {
            error_log($e->getMessage());
            //
        }

        return ob_get_clean();

    }

    public function get_base_url($bucket, $Region='', $Keyname=''){

        $zoneList = get_option('nou_leopard_offload_media_bunny_zone_list');
        $url = '';
        $Bucket_Selected = get_option( 'nou_leopard_offload_media_connection_bucket_selected_select', '' );

        foreach($zoneList as $zone){
            if($zone['Id'] == $Bucket_Selected){
                $hostname = $zone['Hostnames'][0]['Value'];
                $url = "https://{$hostname}";
                break;
            }
        }
        
        return $url;
    }

    public function getObjectUrl( $Bucket = '', $Region = '', $File_Name ) {
        $url = get_option('nou_leopard_offload_media_aws_connection_bucket_base_url');
        return "{$url}/{$File_Name}";
    }

    public static function docs_link_credentials(){
        return 'https://bunnycdn.com/dashboard/account';
    }

    public static function docs_link_create_bucket(){
        return 'https://bunnycdn.com/dashboard/storagezones';
    }

    public function Upload_Media_File( $Bucket='', $Region='', $array_files, $basedir_absolute, $private_or_public = 'public', $prefix='', $attachment_id = 0 ) {

        $base_folder = array_shift( $array_files );

		$files_to_remove = array();

        $S3_Client = $this->Init_S3_Client( '', '', $this->_key );

        $File_Name = array_shift( $array_files );

        if ( $base_folder != '' ) {
            $Key = $base_folder . "/" . $File_Name;
        } else {
            $Key = $File_Name;
        }

        if ( $base_folder != '' ) {
            $SourceFile = $basedir_absolute . "/" . $base_folder . "/" . $File_Name;
        } else {
            $SourceFile = $basedir_absolute . "/" . $File_Name;
        }

        /*== We check if the file is going to be private or public ==*/
        $private_or_public = ( $private_or_public == 'private' ? 'authenticatedRead' : 'publicRead' );

        $base_url = get_option('nou_leopard_offload_media_aws_connection_bucket_base_url');
        $result = false; 
        $bucket = $S3_Client->Storage($this->_storage_key, get_option('nou_leopard_offload_media_bunny_region', 'de'));

        try {
            if(file_exists($SourceFile)){
                $object = $bucket->PutFile($SourceFile, $this->_storage_path, $this->rebuild_key($Key, $prefix));
                if($object['status'] === 'success'){
                    $result = $base_url.'/'.$this->rebuild_key($Key, $prefix);
                    $files_to_remove[] = $SourceFile;
                }else{
                    $msg = $object['msg']->Message;
                    Leopard_Wordpress_Offload_Media_Messages::add_error($msg);
                }
            }
        } catch (\Exception $ex) {
            error_log( $ex->getMessage() );
        }

        foreach ( $array_files as $File_Name ) {

            if ( $base_folder != '' ) {
                $Key = $base_folder . "/" . $File_Name;
            } else {
                $Key = $File_Name;
            }

            if ( $base_folder != '' ) {
                $SourceFile = $basedir_absolute . "/" . $base_folder . "/" . $File_Name;
            } else {
                $SourceFile = $basedir_absolute . "/" . $File_Name;
            }

            try {
                if(file_exists($SourceFile)){
                    $object = $bucket->PutFile($SourceFile, $this->_storage_path, $this->rebuild_key($Key, $prefix));
                    if($object['status'] === 'success'){
                        $files_to_remove[] = $SourceFile;
                    }
                }
            } catch (\Exception $ex) {
                error_log( $ex->getMessage() );
            }
        }

        $files_to_remove = array_unique( $files_to_remove );
        $this->remove_local_files($files_to_remove, $attachment_id);
        
        return $result;

    }

    /**
     * elimina un objeto de un bucket
     *
     * @param $key
     */
    public function deleteObject_nou( $Bucket='', $Region='', $array_files ) {

        $base_folder = array_shift( $array_files );

        $S3_Client = $this->Init_S3_Client( '', '', $this->_key );

        $result = 0;
        $bucket = $S3_Client->Storage($this->_storage_key, get_option('nou_leopard_offload_media_bunny_region', 'de'));

        foreach ( $array_files as $File_Name ) {

            if ( $base_folder != '' ) {
                $Key = $base_folder . "/" . $File_Name;
            } else {
                $Key = $File_Name;
            }

            try {
                $result = $bucket->DeleteFile($this->_storage_path.$this->rebuild_key($Key));
            } catch (\Throwable $th) {
                error_log($th->getMessage());
            }
            
            $enable_webp = get_option('nou_leopard_offload_media_webp');
            if($enable_webp){
                try {
                    if(strpos($Key, '.png') !== false || strpos($Key, '.jpg') !== false || strpos($Key, '.jpeg') !== false){
                        $filename = basename($Key);
				        $newKey = str_replace($filename, sanitize_title($filename), $Key);
                        $bucket->DeleteFile($this->_storage_path.$this->rebuild_key($newKey.'.webp'));
                    }
                } catch (Exception $e) {
                    error_log($e->getMessage());
                }
            }

        }

        return $result;
    }

    /**
     * @param $key
     *
     * @return \Guzzle\Service\Resource\Model
     */
    public function getObject( $Bucket = '', $Region = '', $key = '', $expires = null ) {
        
        $S3_Client = $this->Init_S3_Client( '', '', $this->_key );
        $bucket = $S3_Client->Storage($this->_storage_key, get_option('nou_leopard_offload_media_bunny_region', 'de'));

        $data = array(
            'Bucket' => $Bucket,
            'Key'    => $this->rebuild_key($key)
        );
        
        return [];
    }

    public function downloadObject($data)
    {
        if(!file_exists($data['SaveAs'])){
            // If the function it's not available, require it.
            if ( ! function_exists( 'download_url' ) ) {
                require_once ABSPATH . 'wp-admin/includes/file.php';
            }
            
            $base_url = get_option('nou_leopard_offload_media_aws_connection_bucket_base_url');
            $url = $base_url.'/'.$data['Key'];
            $tmp_file = download_url( $url );

            // Copies the file to the final destination and deletes temporary file.

            copy( $tmp_file, $data['SaveAs'] );
            @unlink( $tmp_file );
        }
    }

    /**
     * download files
     *
     * @param $key
     * @param $filename
     */
    public function download_file( $Bucket, $Region, $array_files, $basedir_absolute ) {
        $result = false;

        $base_folder = array_shift( $array_files );
        foreach ( $array_files as $File_Name ) {

            if ( $base_folder != '' ) {
                $Key = $base_folder . "/" . $File_Name;
            } else {
                $Key = $File_Name;
            }

            if ( $base_folder != '' ) {
                $SaveAs = $basedir_absolute . "/" . $base_folder . "/" . $File_Name;
            } else {
                $SaveAs = $basedir_absolute . "/" . $File_Name;
            }

            try{
                $dir = dirname( $SaveAs );
                if ( ! wp_mkdir_p( $dir ) ) {
                    $error_message = sprintf( __( 'The local directory %s does not exist and could not be created.', 'leopard-wordpress-offload-media' ), $dir );
                    error_log( sprintf( __( 'There was an error attempting to download the file %s from the bucket: %s', 'leopard-wordpress-offload-media' ), $File_Name, $error_message ) );

                    return false;
                }
                
                $result = $this->downloadObject( array(
                    'Key'    => $this->rebuild_key($Key),
                    'SaveAs' => $SaveAs
                ) );
            } catch(Exception $e) {
                error_log($e->getMessage());
            }

        }

        return $result;

    }

    public function registerStreamWrapper()
    {
        return;
    }

    public function update_permission()
    {
        return;
    }

    public function set_object_permission($Bucket, $Region, $array_files, $acl='public-read')
    {
        return;
    }

    public function putFileContent($Bucket = '', $Region = '', $path, $content){
        return file_put_contents($path, $content);
    }

    public function uploadSingleFile($Bucket = '', $Region = '', $SourceFile, $Key){
        $S3_Client = $this->Init_S3_Client( '', '', $this->_key );
        $base_url = get_option('nou_leopard_offload_media_aws_connection_bucket_base_url');
        $result = false; 
        $bucket = $S3_Client->Storage($this->_storage_key, get_option('nou_leopard_offload_media_bunny_region', 'de'));
        try {
            if(file_exists($SourceFile)){
                $object = $bucket->PutFile($SourceFile, $this->_storage_path, $this->rebuild_key($Key));
                return $base_url.'/'.$this->rebuild_key($Key, '');
            }
        } catch (\Exception $ex) {
            error_log( $ex->getMessage() );
        }

        return $result;
    }

    public function putHostingContent($region='', $bucket='')
    {
        return null;
    }

    public function putBucketPolicy($region='', $bucket='')
    {
        return null;
    }

    public function Get_Presigned_URL( $Bucket, $Region, $Key ) {
        $S3_Client = $this->Init_S3_Client( '', '', $this->_key );
        $path = $this->rebuild_key($Key);
        return (string) $S3_Client->SecureLink($path);
    }
}