<?php
if (!defined('ABSPATH')) {exit;}

/**
 * Remove files bucket Background Process
 *
 * @link       https://themeforest.net/user/nouthemes/portfolio
 * @since      2.0
 *
 * @package    Leopard_Wordpress_Offload_Media
 * @subpackage Leopard_Wordpress_Offload_Media/includes
 */

class Leopard_Wordpress_Offload_Media_Remove_Files_Bucket_Process {

	/**
	 * Initiate new background process.
	 */
	public function __construct() {
		if(!$this->check_action()){
			$this->unschedule();
		}else{
			add_action( 'nou_leopard_offload_media_cronjob_remove_files_from_bucket', [ $this, 'task' ] );
		}
	}

	public function check_action(){
		$action_scan = get_option('nou_leopard_offload_media_action');
		if($action_scan == 'remove_files_from_bucket'){
			return true;
		}

		return false;
	}

	public function unschedule(){
		if ( function_exists( 'as_next_scheduled_action' ) ) {
			leopard_wordpress_offload_media_unschedule_action( 'nou_leopard_offload_media_cronjob_remove_files_from_bucket' );
		}

		return false;
	}

	/**
	 * Complete
	 */
	public function complete() {
		leopard_offload_media_after_action_scheduler_completed();
	}

	public function clearScheduled()
	{
		try {
			if ( function_exists( 'as_next_scheduled_action' ) ) {
				leopard_wordpress_offload_media_unschedule_action( 'nou_leopard_offload_media_cronjob_scan_attachments' );
			}
		} catch (\Throwable $th) {}

		try {
			if ( function_exists( 'as_next_scheduled_action' ) ) {
				leopard_wordpress_offload_media_unschedule_action( 'nou_leopard_offload_media_cronjob_remove_files_from_server' );
			}
		} catch (\Throwable $th) {}

		try {
			if ( function_exists( 'as_next_scheduled_action' ) ) {
				leopard_wordpress_offload_media_unschedule_action( 'nou_leopard_offload_media_cronjob_copy_attachments_to_cloud' );
			}
		} catch (\Throwable $th) {}

		try {
			if ( function_exists( 'as_next_scheduled_action' ) ) {
				leopard_wordpress_offload_media_unschedule_action( 'nou_leopard_offload_media_cronjob_download_files_from_bucket' );
			}
		} catch (\Throwable $th) {}
	}

	/**
	 * Task
	 * 
	 * @return mixed
	 */
	public function task() {

		$this->clearScheduled();

		if(!leopard_wordpress_offload_media_cronjob_timed()){
			return false;
		}

		$status = get_option('nou_leopard_offload_media_remove_files_from_bucket_status', 0);
		if($status == 0){
			return false;
		}

		$total_synced = get_option('nou_leopard_offload_media_processed_removed_files_from_bucket', []);
		if(!is_array($total_synced)){
			$total_synced = [];
		}

		$totalItems = get_option('nou_leopard_offload_media_scanned_attachments', []);
		if(count($totalItems) == 0){
			$this->unschedule();
		}
		
		$items = get_option('nou_leopard_offload_media_scanned_attachments_copy', []);
		if(count($items) == 0){
			$items = $totalItems;
			update_option('nou_leopard_offload_media_scanned_attachments_copy', $totalItems);
		}
		$item = $items[0];

		update_option('nou_leopard_offload_media_step_scan_attachments', 2);
		
		array_splice($items, 0, 1);
		
		update_option('nou_leopard_offload_media_scanned_attachments_copy', $items);

		$outputProgress = get_option('nou_leopard_offload_media_output_progress', []);

		$path = get_post_meta($item, '_wp_attached_file', true);
		if(!empty($path)){
			$outputProgress[] = $path;
			update_option('nou_leopard_offload_media_output_progress', array_unique($outputProgress));
		}

		$total_synced[] = $item;
		update_option('nou_leopard_offload_media_processed_removed_files_from_bucket', array_unique($total_synced));
		
		try{
			leopard_offload_media_remove_from_s3_function($item);
		}catch(Exception $e){
			error_log($e->getMessage());
		}
		
		if(count($items) == 0){
			$this->complete();
		}

		return false;
	}
}
?>